// Hero.cpp

#include "Hero.h"
#include "Bullet.h"
#include "LogManager.h"
#include "InputManager.h"
#include "ResourceManager.h"
#include "WorldManager.h"
#include "EventStep.h"
#include "GameManager.h"
#include "EventNuke.h"
#include "GameOver.h"
#include "Explosion.h"
#include "GraphicsManager.h"

bool slowDownEnemies = false;
bool burstFire = false;

Hero::Hero() {

  // link to "ship" sprite
  ResourceManager &rm = ResourceManager::getInstance();
  LogManager &lm = LogManager::getInstance();
  Sprite *p_temp_sprite;
  p_temp_sprite = rm.getSprite("ship");
  if (!p_temp_sprite) {
    lm.writeLog("Hero::Hero(): Warning! Sprite '%s' not found", "ship");
    return;
  }
  setSprite(p_temp_sprite, true); // center and define box
  
  setSpriteSlowdown(3);		  // third speed animation

  // player controls hero, so register with keyboard
  InputManager &im = InputManager::getInstance();
  im.registerInterest(this, KEYBOARD_EVENT);

  // need to update fire rate control each step
  WorldManager &world_manager = WorldManager::getInstance();
  world_manager.registerInterest(this, STEP_EVENT);

  setType("Hero");

  // set starting location
  pos.setX(7);
  pos.setY(world_manager.getBoundary().getVertical()/2);

  fire_slowdown = 15;
  fire_countdown = fire_slowdown;

  nukeCount = 1;

 
}

Hero::~Hero() {
  InputManager &im = InputManager::getInstance();
  im.unregisterInterest(this, KEYBOARD_EVENT);
  WorldManager &world_manager = WorldManager::getInstance();
  world_manager.unregisterInterest(this, STEP_EVENT);

  string temp = "gameover";
  GameOver *p_go = new GameOver(temp);

  // make big explosion
  for (int i=-8; i<=8; i+=5) {
    for (int j=-5; j<=5; j+=3) {
      Position temp_pos = this->getPos();
      temp_pos.setX(this->getPos().getX() + i);
      temp_pos.setY(this->getPos().getY() + j);
      Explosion *p_explosion = new Explosion;
      p_explosion -> setPos(temp_pos);
    }
  }

  //Why not blow up all the other ships too for effect
  nukeCount = 1;
  nuke();

}

// handle event
// return 0 if ignored, else 1
int Hero::eventHandler(Event *e) 
{
  if (e->getType() == KEYBOARD_EVENT) {
    EventKeyboard *p_keyboard_event = static_cast <EventKeyboard *> (e);
    kbd(p_keyboard_event);
    return 1;
  }

  if (e->getType() == STEP_EVENT) {
    step();
    return 1;
  }

  // if we get here, we have ignored this event
  return 0;
}

// call move (or do nothing) according to key pressed
void Hero::kbd(EventKeyboard *p_keyboard_event) {

  switch(p_keyboard_event->getKey()) {
  case KEY_UP:			// up arrow
    move(-1);
    break;
  case KEY_DOWN:		// down arrow
    move(+1);
    break;
  case ' ':			// fire
	fire();
    break;
  case 13:			// nuke
    nuke();
    break;
  case 113: // q = 5 more nukes
	nukeCount += 5;
	break;
  case 119: // w = slow down enemies cheat
	slowDownEnemies = true;
	break;
  case 101: // e = burst fire cheat
    burstFire = true;
	break;
  };
  return;
}

// move up or down
void Hero::move(int dy) {
  WorldManager &world_manager = WorldManager::getInstance();
  Position new_pos(pos.getX(), pos.getY() + dy);

  // if stays on screen, allow move
  if ((new_pos.getY() > 0) && 
      (new_pos.getY() < world_manager.getBoundary().getVertical()))
    world_manager.moveObj(this, new_pos);
}

// fire a bullet
void Hero::fire() {
  if (fire_countdown > 0)
    return;
  fire_countdown = fire_slowdown;
  if(burstFire == false) // if no cheat is on, fire normal bullet
  {
	new Bullet(pos);
  }
  else // if cheat is on...
  {
	 Position upper_bullet(pos.getX(), pos.getY() + 2); //create 2 new bullet positions
	 Position lower_bullet(pos.getX(), pos.getY() - 2);
	 
	 new Bullet(pos);
     new Bullet(upper_bullet);
	 new Bullet(lower_bullet);
  }
}

// decrease fire restriction
void Hero::step() {
  fire_countdown--;
  if (fire_countdown < 0)
    fire_countdown = 0;
}

void Hero::nuke() {
  
  if(nukeCount > 0)
  {
	  // create a "nuke" event and send to all interested objects
	 WorldManager &world_manager = WorldManager::getInstance();
	 EventNuke nuke;
	 world_manager.onEvent(&nuke);

	  nukeCount--;
  }
  else
  {
	  //do nothing
  }
}

